package com.danitheskunk.skunkworks;

import static org.lwjgl.opengl.GL11.*;

class GLRenderContext extends BaseRenderContext implements IRenderContext{
	private final GLTextureAtlas atlas;

	public GLRenderContext(GLTextureAtlas atlas) {
		this.atlas = atlas;
	}

	@Override
	public void drawNineSlice(NineSlice slice, Recti rect) {
		var centerWidth = Math.min(0, rect.getWidth() - slice.getLeft().getSize().getX() - slice.getRight().getSize().getX());
		var centerHeight = Math.min(0, rect.getHeight() - slice.getTop().getSize().getY() - slice.getBottom().getSize().getY());
		var pos = rect.getPos();
		var size = rect.getSize();

		drawTexture(pos, slice.getTopLeft());
	}

	@Override
	public void drawRectangle(Recti rect, Color color) {
		var tl = rect.getTopLeft();
		var tr = rect.getTopRight();
		var bl = rect.getBottomLeft();
		var br = rect.getBottomRight();

		//todo: make work with enabled textures

		glColor4f(
			color.getR() / 255.0f,
			color.getG() / 255.0f,
			color.getB() / 255.0f,
			color.getA() / 255.0f
		);

		//counterclockwise triangles
		glVertex2i(bl.getX(), bl.getY());
		glVertex2i(tr.getX(), tr.getY());
		glVertex2i(tl.getX(), tl.getY());

		glVertex2i(tr.getX(), tr.getY());
		glVertex2i(bl.getX(), bl.getY());
		glVertex2i(br.getX(), bl.getY());
	}

	@Override
	public void drawTexture(Vec2i pos, ITexture texture) {
		drawTextureRectangle(new Recti(pos, texture.getSize()), texture);
	}

	@Override
	public void drawTexture(Vec2i pos, ITexture texture, Color color) {
		drawTextureRectangle(new Recti(pos, texture.getSize()), texture, color);
	}

	@Override
	public void drawTextureRectangle(Recti rect, ITexture texture) {
		drawTextureRectangle(rect, texture, Color.WHITE);
	}

	@Override
	public void drawTextureRectangle(Recti rect, ITexture texture, Color color) {
		var tex = (GLTexture)texture;

		var tl = rect.getTopLeft();
		var tr = rect.getTopRight();
		var bl = rect.getBottomLeft();
		var br = rect.getBottomRight();

		var ttl = Vec2i.divf(tex.getTexArea().getTopLeft(),     atlas.getSize());
		var ttr = Vec2i.divf(tex.getTexArea().getTopRight(),    atlas.getSize());
		var tbl = Vec2i.divf(tex.getTexArea().getBottomLeft(),  atlas.getSize());
		var tbr = Vec2i.divf(tex.getTexArea().getBottomRight(), atlas.getSize());

		glColor4f(
			color.getR() / 255.0f,
			color.getG() / 255.0f,
			color.getB() / 255.0f,
			color.getA() / 255.0f
		);

		//counterclockwise triangles
		glTexCoord2d(tbl.getX(), tbl.getY());
		glVertex2i(bl.getX(), bl.getY());
		glTexCoord2d(ttr.getX(), ttr.getY());
		glVertex2i(tr.getX(), tr.getY());
		glTexCoord2d(ttl.getX(), ttl.getY());
		glVertex2i(tl.getX(), tl.getY());

		glTexCoord2d(ttr.getX(), ttr.getY());
		glVertex2i(tr.getX(), tr.getY());
		glTexCoord2d(tbl.getX(), tbl.getY());
		glVertex2i(bl.getX(), bl.getY());
		glTexCoord2d(tbr.getX(), tbr.getY());
		glVertex2i(br.getX(), bl.getY());
	}
}
